<?php

function xc_default_xc_search_block_label($field_label, $label, $is_active) {
  if ($is_active) {
    return ( $field_label ? '<span class="xc-search-facet-field-label">' . $field_label . ': </span>' : '' )
    . '<span class="xc-search-facet-label">' . $label . '</span>';
  }
  else {
    return ( $field_label ? $field_label . ': ' : '' ) . $label;
  }
}

/**
 * Override the xc_search_info_bar_back_to_results theme
 *
 * @param $meta (Object)
 *   The meta object, which contains information about the search result
 */
function xc_default_xc_search_info_bar_back_to_results($meta) {
  timer_start('xc_default_xc_search_info_bar_back_to_results');
  $results_page = $meta->extra['list'];

  // get image
  $image = theme('image',
    drupal_get_path('theme', 'xc_theme') . '/xc_default/images/arrow-back.png',
    $results_page['label'], $results_page['label']
  );

  // link options
  $options = array(
    'html' => TRUE,
    'query' => $results_page['query'],
    'fragment' => $results_page['fragment']
  );

  $image = l($image, $results_page['url'], $options);
  $label = l($results_page['label'], $results_page['url'], $options);

  $output = '<div class="xc-search-back-to-results">' . $image  . ' ' . $label . '</div>';

  timer_stop('xc_default_xc_search_info_bar_back_to_results');
  return $output;
}

/**
 * Theming the auto login form
 *
 * @param $form (Array)
 *   The form object
 */
function xc_default_xc_auth_auto_login_form($form) {
  $output = '';
  foreach (element_children($form) as $id => $key) {
    if ($key == 'submit') {
      $submit = drupal_render($form[$key]);
    }
    elseif ($key == 'reset_password_text') {
      $text = drupal_render($form[$key]);
    }
    else {
      $output .= drupal_render($form[$key]);
    }
  }
  $output .= '<div id="xc-login-text-and-button">'
    . '<span id="xc-login-button">' . $submit . '</span>'
    . $text
    . '</div>';
  $output .= drupal_render($form);
  return $output;
}

/**
 * Theming search info bar with remove icon
 */
function xc_default_xc_search_info_bar_actions($links, $buttons) {
  $action_links = array();
  $action_buttons = array();

  foreach ($links as $class => $title) {
    $action_links[] = array(
      'href' => '',
      'title' => $title,
      'attributes' => array('class' => $class),
      'fragment' => ' ',
      'external' => TRUE,
    );
  }

  foreach ($buttons as $class => $properties) {
    $attributes = array('class' => $class);
    $id = $class;

    if (!is_array($properties)) {
      $title = $properties;
    }
    else {
      $title = $properties['title'];
      if (isset($properties['#disabled']) && $properties['#disabled']) {
        $attributes['class'] .= ' action-disabled';
        $attributes['disabled'] = 'disabled';
      }
      if (isset($properties['#id'])) {
        $id = $properties['#id'];
      }
    }

    $definition = array(
      '#value' => $title,
      '#button_type' => 'submit',
      '#attributes' => $attributes
    );

    if (isset($id) && !is_null($id)) {
      $definition['#id'] = $id;
    }

    $data = theme('button', $definition);
    $action_buttons[]['data'] = $data;
  }

  $distinct_bars = FALSE;
  $last_button = end($action_buttons);
  if (preg_match('/id="remove-bookmark"/', $last_button['data'])) {
    $distinct_bars = TRUE;
    array_pop($action_buttons); // remove last button
    $remove_bar = array($last_button);
  }

  $output = '<div class="xc-search-actions">'
    . '<span class="xc-search-action-links">' . t('Select: ') . theme('links', $action_links) . '</span>'
    . '<span class="xc-search-action-buttons">' . theme('item_list', $action_buttons) . '</span>';
  if ($distinct_bars) {
    $output .= '<span class="xc-search-action-buttons xc-remove-bar">'
            . '<img class="xc-remove-delete-icon" src="' . url(drupal_get_path('theme', 'xc_default'), array('external' => TRUE)) . '/images/remove-icon.png' . '"/>'
            . theme('item_list', $remove_bar) . '</span>';
  }
  $output .= '</div>';

  return $output;
}

